@@ -187,7 +187,7 @@ TEST(FeatureResolverTest, DefaultsGeneratedPoolCustom) {
           pool.FindMessageTypeByName("google.protobuf.FeatureSet"),
           {pool.FindExtensionByName("pb.test")}, EDITION_2023, EDITION_2023);
   ASSERT_OK(defaults);
-  ASSERT_EQ(defaults->defaults().size(), 3);
+  ASSERT_EQ(defaults->defaults().size(), 4);
   ASSERT_EQ(defaults->defaults().at(2).edition(), EDITION_2023);
   FeatureSet merged = defaults->defaults().at(2).overridable_features();
 
@@ -203,7 +203,7 @@ TEST(FeatureResolverTest, DefaultsMergedFeatures) {
                                        {GetExtension(pb::test)}, EDITION_2023,
                                        EDITION_2023);
   ASSERT_OK(defaults);
-  ASSERT_EQ(defaults->defaults_size(), 3);
+  ASSERT_EQ(defaults->defaults_size(), 4);
 
   defaults->mutable_defaults(2)
       ->mutable_fixed_features()
@@ -269,7 +269,7 @@ TEST(FeatureResolverTest, CompileDefaultsFixedFutureFeature) {
                                        {GetExtension(pb::test)}, EDITION_PROTO2,
                                        EDITION_2023);
   ASSERT_OK(defaults);
-  ASSERT_EQ(defaults->defaults_size(), 3);
+  ASSERT_EQ(defaults->defaults_size(), 4);
 
   const auto& edition_defaults = defaults->defaults(2);
   ASSERT_EQ(edition_defaults.edition(), EDITION_2023);
@@ -291,7 +291,7 @@ TEST(FeatureResolverTest, CompileDefaultsFixedRemovedFeature) {
                                        {GetExtension(pb::test)}, EDITION_PROTO2,
                                        EDITION_2024);
   ASSERT_OK(defaults);
-  ASSERT_EQ(defaults->defaults_size(), 4);
+  ASSERT_EQ(defaults->defaults_size(), 5);
 
   const auto& edition_defaults = defaults->defaults(3);
   ASSERT_EQ(edition_defaults.edition(), EDITION_2024);
@@ -314,7 +314,7 @@ TEST(FeatureResolverTest, CompileDefaultsOverridable) {
                                        {GetExtension(pb::test)}, EDITION_PROTO2,
                                        EDITION_2023);
   ASSERT_OK(defaults);
-  ASSERT_EQ(defaults->defaults_size(), 3);
+  ASSERT_EQ(defaults->defaults_size(), 4);
 
   const auto& edition_defaults = defaults->defaults(2);
   ASSERT_EQ(edition_defaults.edition(), EDITION_2023);
@@ -421,6 +421,14 @@ TEST(FeatureResolverTest, CompileDefaultsMinimumLaterThanMaximum) {
                      HasSubstr("2023"))));
 }
 
+TEST(FeatureResolverTest, CompileDefaultsUnstableLaterThanMaximum) {
+  absl::StatusOr<FeatureSetDefaults> defaults =
+      FeatureResolver::CompileDefaults(FeatureSet::descriptor(), {},
+                                       EDITION_2023, EDITION_2024);
+  ASSERT_OK(defaults);
+  EXPECT_OK(FeatureResolver::Create(EDITION_UNSTABLE, *defaults));
+}
+
 TEST(FeatureResolverTest, MergeFeaturesChildOverrideCore) {
   absl::StatusOr<FeatureResolver> resolver = SetupFeatureResolver(EDITION_2023);
   ASSERT_OK(resolver);
@@ -556,6 +564,8 @@ TEST(FeatureResolverTest, GetEditionFeatureSetDefaults) {
       internal::GetEditionFeatureSetDefaults(EDITION_PROTO3, *defaults);
   absl::StatusOr<FeatureSet> edition_proto2_feature =
       internal::GetEditionFeatureSetDefaults(EDITION_LEGACY, *defaults);
+  absl::StatusOr<FeatureSet> edition_unstable_feature =
+      internal::GetEditionFeatureSetDefaults(EDITION_UNSTABLE, *defaults);
   absl::StatusOr<FeatureSet> edition_test_feature =
       internal::GetEditionFeatureSetDefaults(EDITION_99998_TEST_ONLY,
                                              *defaults);
@@ -568,6 +578,10 @@ TEST(FeatureResolverTest, GetEditionFeatureSetDefaults) {
   EXPECT_OK(edition_proto2_feature);
   EXPECT_EQ(edition_proto2_feature->GetExtension(pb::test).file_feature(),
             pb::VALUE1);
+  EXPECT_OK(edition_unstable_feature);
+  EXPECT_EQ(
+      edition_unstable_feature->GetExtension(pb::test).new_unstable_feature(),
+      pb::UNSTABLE2);
   EXPECT_OK(edition_test_feature);
   EXPECT_EQ(edition_test_feature->GetExtension(pb::test).file_feature(),
             pb::VALUE4);
@@ -594,6 +608,16 @@ TEST(FeatureResolverLifetimesTest, Valid) {
   EXPECT_THAT(results.warnings, IsEmpty());
 }
 
+TEST(FeatureResolverLifetimesTest, ValidUnstableFeature) {
+  FeatureSet features = ParseTextOrDie(R"pb(
+    [pb.test] { new_unstable_feature: UNSTABLE2 }
+  )pb");
+  auto results = FeatureResolver::ValidateFeatureLifetimes(EDITION_UNSTABLE,
+                                                           features, nullptr);
+  EXPECT_THAT(results.errors, IsEmpty());
+  EXPECT_THAT(results.warnings, IsEmpty());
+}
+
 TEST(FeatureResolverLifetimesTest, DeprecatedFeature) {
   FeatureSet features = ParseTextOrDie(R"pb(
     [pb.test] { removed_feature: VALUE1 }
@@ -620,6 +644,19 @@ TEST(FeatureResolverLifetimesTest, RemovedFeature) {
   EXPECT_THAT(results.warnings, IsEmpty());
 }
 
+TEST(FeatureResolverLifetimesTest, RemovedUnstableFeature) {
+  FeatureSet features = ParseTextOrDie(R"pb(
+    [pb.test] { removed_unstable_feature: UNSTABLE1 }
+  )pb");
+  auto results = FeatureResolver::ValidateFeatureLifetimes(EDITION_UNSTABLE,
+                                                           features, nullptr);
+  EXPECT_THAT(
+      results.errors,
+      ElementsAre(AllOf(HasSubstr("pb.TestFeatures.removed_unstable_feature"),
+                        HasSubstr("removed in edition UNSTABLE"))));
+  EXPECT_THAT(results.warnings, IsEmpty());
+}
+
 TEST(FeatureResolverLifetimesTest, NotIntroduced) {
   FeatureSet features = ParseTextOrDie(R"pb(
     [pb.test] { future_feature: VALUE1 }
@@ -716,6 +753,34 @@ TEST(FeatureResolverLifetimesTest, ValueSupportBeforeIntroduced) {
   EXPECT_THAT(results.warnings, IsEmpty());
 }
 
+TEST(FeatureResolverLifetimesTest, ValueSupportBeforeIntroducedUnstable) {
+  FeatureSet features = ParseTextOrDie(R"pb(
+    [pb.test] { unstable_existing_feature: UNSTABLE3 }
+  )pb");
+  auto results = FeatureResolver::ValidateFeatureLifetimes(EDITION_2023,
+                                                           features, nullptr);
+  EXPECT_THAT(
+      results.errors,
+      ElementsAre(AllOf(HasSubstr("pb.UNSTABLE3"),
+                        HasSubstr("wasn't introduced until edition UNSTABLE"),
+                        HasSubstr("can't be used in edition 2023"))));
+  EXPECT_THAT(results.warnings, IsEmpty());
+}
+
+TEST(FeatureResolverLifetimesTest, FeatureSupportBeforeIntroducedUnstable) {
+  FeatureSet features = ParseTextOrDie(R"pb(
+    [pb.test] { new_unstable_feature: UNSTABLE2 }
+  )pb");
+  auto results = FeatureResolver::ValidateFeatureLifetimes(EDITION_2023,
+                                                           features, nullptr);
+  EXPECT_THAT(
+      results.errors,
+      ElementsAre(AllOf(HasSubstr("pb.TestFeatures.new_unstable_feature "),
+                        HasSubstr("wasn't introduced until edition UNSTABLE"),
+                        HasSubstr("can't be used in edition 2023"))));
+  EXPECT_THAT(results.warnings, IsEmpty());
+}
+
 TEST(FeatureResolverLifetimesTest, ValueSupportAfterRemoved) {
   FeatureSet features = ParseTextOrDie(R"pb(
     [pb.test] { value_lifetime_feature: VALUE_LIFETIME_REMOVED }
